// @(#)$Id: ListIterator.spec,v 1.12 2004/06/02 17:41:49 leavens Exp $

// Copyright (C) 2001, 2004 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


package java.util;

//@ model import org.jmlspecs.models.*;

/** JML's specification of java.util.Iterator.
 * Some of this specification is taken from ESC/Java.
 * @version $Revision: 1.12 $
 * @author Gary T. Leavens
 */
public interface ListIterator extends Iterator {

    //@ model instance int cursor_position; in objectState;

    //@ public instance invariant 0 <= cursor_position;

    /*@ public model instance JMLEqualsSequence previousElements;
      @                                              in cursor_position;
      @ public initially previousElements.isEmpty();
      @ public instance invariant previousElements != null;
      @*/

    //@ public instance represents cursor_position <- previousElements.size();

    /*@ pure @*/ boolean hasNext();

    /*@ also
      @ public normal_behavior
      @   requires hasNext();
      @   assignable objectState, remove_called_since, moreElements;
      @   ensures previousElements.equals(
      @          \old(previousElements).insertBack(\result));
      @*/
    Object next();

    /*@ public normal_behavior
      @   ensures \result <==> !previousElements.isEmpty();
      @*/
    /*@ pure @*/ boolean hasPrevious();

    /*@ public normal_behavior
      @   requires hasPrevious();
      @   assignable objectState, previousElements, remove_called_since;
      @   ensures \result == \old(previousElements.last())
      @        && previousElements.equals(\old(previousElements.header()));
      @   ensures !remove_called_since;
      @*/
    Object previous();

    /*@ public normal_behavior
      @   ensures \result == previousElements.size();
      @   ensures_redundantly \result == cursor_position;
      @*/
    /*@ pure @*/ int nextIndex();

    /*@ public normal_behavior
      @   ensures \result == previousElements.size() - 1;
      @   ensures_redundantly \result == cursor_position - 1;
      @*/
    /*@ pure @*/ int previousIndex();


    // Modification Operations
    
    /*@ also
      @   assignable objectState, remove_called_since;
      @   assignable previousElements;
      @   ensures previousElements.equals(previousElements.header());
      @   ensures_redundantly previousElements.size()
      @                       == \old(previousElements.size() - 1);
      @   ensures_redundantly cursor_position == \old(cursor_position - 1);
      @*/
    void remove();

    /*@ assignable objectState, previousElements;
      @ ensures previousElements.equals(
      @                  \old(previousElements.header().insertBack(o)));
      @ signals (UnsupportedOperationException);
      @ signals (ClassCastException);
      @ signals (IllegalArgumentException);
      @ signals (IllegalStateException) \old(remove_called_since);
      @*/
    void set(Object o);

    /*@ assignable objectState, previousElements,
      @            remove_called_since;
      @ ensures remove_called_since;
      @ ensures previousElements.equals(
      @                  \old(previousElements.insertBack(o)));
      @ signals (UnsupportedOperationException);
      @ signals (ClassCastException);
      @ signals (IllegalArgumentException);
      @*/
    void add(Object o);
}
