// @(#)$Id: Set.spec,v 1.17 2004/06/01 23:25:49 leavens Exp $

// Adapted in part from Compaq SRC's specification for ESC/Java

// Copyright (C) 2000, 2002 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package java.util;

//@ model import org.jmlspecs.models.*;

/** JML's specification of java.util.Set.
 * @version $Revision: 1.17 $
 * @author Brandon Shilling
 * @author Gary T. Leavens
 */
public interface Set extends Collection {

    /*@ public model instance non_null JMLEqualsSet theSet;
      @                                               in theCollection;
      @ public instance represents theCollection <- theSet.toBag();
      @*/

    // specification inherited
    //@ pure
    int size();

    // specification inherited
    //@ pure
    boolean isEmpty();

    // specification inherited
    //@ pure
    boolean contains(Object o);

    // specification inherited
    //@ pure
    Iterator iterator();

    // specification inherited
    //@ pure
    Object[] toArray();

    // specification inherited
    Object[] toArray(Object[] a);

    /*@ also
       @ public behavior
       @   requires !theSet.has(o);
       @   assignable theCollection;
       @   ensures \result <==> !\old(theSet).has(o);
       @   ensures theSet.equals(\old(theSet).insert(o));
       @*/
    boolean add(Object o);

    /*@ also
       @ implies_that
       @ public behavior
       @   assignable theCollection;
       @   assignable_redundantly theSet;
       @   ensures !theSet.has(o);
       @   ensures_redundantly !theCollection.has(o);
       @*/
    boolean remove(Object o);

    // Bulk Operations

    /*@ also
       @ public behavior
       @   requires c instanceof Set;
       @   ensures \result ==> ((Set) c).theSet.isSubset(theSet);
       @*/
    //@ pure
    boolean containsAll(Collection c);

    /*@ also
       @ public behavior
       @   assignable theCollection;
       @   ensures 
       @        theSet.equals(\old(theSet).union(JMLEqualsSet.convertFrom(c)))
       @        && (\result ==> !theSet.equals(\old(theSet)));
       @*/
    boolean addAll(Collection c);

    /*@ also
       @ public behavior
       @   assignable theCollection;
       @   ensures theSet.equals(\old(theSet).intersection(
       @                                  JMLEqualsSet.convertFrom(c)))
       @        && (\result ==> !theSet.equals(\old(theSet)));
       @*/
    boolean retainAll(Collection c);

    /*@ also
       @ public behavior
       @   assignable theCollection;
       @   ensures theSet.equals(\old(theSet).difference(
       @                                  JMLEqualsSet.convertFrom(c)))
       @        && (\result ==> !theSet.equals(\old(theSet)));
       @*/
    boolean removeAll(Collection c);

    // specification inherited
    void clear();

    // Comparison and hashing

    /*@ also
       @ public normal_behavior
       @   requires o instanceof Set;
       @   ensures \result ==> theSet.equals(((Set)o).theSet);
       @*/
    /*@ pure @*/ boolean equals(Object o);

    // specification inherited
    int hashCode();
}
