// Copyright (C) 2003 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with GNU Emacs; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

package java.util;

//@ model import org.jmlspecs.models.*;

/** JML's specification of java.util.SortedSet.
 * @version $Revision: 1.6 $
 * @author Katie Becker
 * @author Gary T. Leavens
 */
public /*@ pure @*/ interface SortedSet extends Set {

    //@ public model instance JMLType firstElement;
    //@ public model instance JMLType lastElement;
    
    /*@ public normal_behavior
      @    ensures true;
      @*/  
    /*@ pure @*/ Comparator comparator();

    /*@ public behavior
      @    ensures \result != null;
      @ //!FIXME! add in the following when can do JMLEqualsSet comprehensions
      @ //        && \result.theSet.equals(
      @ //                new JMLEqualsSet{ Object o | this.has(o) && o != null
      @ //                     && 0 <= comparator().compare(fromElement, o)
      @ //                     && comparator().compare(o, toElement) > 0});
      @    signals (ClassCastException)
      @           (* \typeof(fromElement) or \typeof(toElement)
      @            is incompatible with this set's comparator *); 
      @    signals (IllegalArgumentException) 
      @           (* fromElement > toElement || fromElement or 
      @            toElement is not within the domain of 
      @            the SortedSet *); 
      @    signals (NullPointerException) 
      @            (fromElement==null || toElement==null) 
      @             && !containsNull;
      @*/
    SortedSet subSet(Object fromElement, Object toElement); 

    /*@ public behavior
      @    ensures (\result.firstElement.equals(firstElement));
      @    ensures \result != null;
      @    signals (ClassCastException)
      @            (* \typeof(toElement) is incompatible with
      @             with this set's comparator *); 
      @    signals (IllegalArgumentException) 
      @            (* toElement is not within the domain of 
      @             the SortedSet *); 
      @    signals (NullPointerException) toElement==null && !containsNull;
      @*/
    SortedSet headSet(Object toElement); 
    
    /*@ public behavior
      @    ensures \result != null;
      @ //!FIXME! specify more of the behavior
      @    signals (ClassCastException)
      @           (* \typeof(fromElement) is incompatible with this
      @            set's comparator *); 
      @    signals (IllegalArgumentException) 
      @           (* fromKey is not within the domain of 
      @            the SortedSet *); 
      @    signals (NullPointerException) fromElement==null 
      @             && !containsNull;
      @*/ 
    SortedSet tailSet(Object fromElement); 

    /*@ public behavior
      @    ensures \result.equals(firstElement);
      @    signals (NoSuchElementException) theSet.isEmpty();
      @*/
    Object first(); 
   
    /*@ public behavior
      @    ensures \result.equals(lastElement);
      @    signals (NoSuchElementException) theSet.isEmpty();
      @*/
    Object last(); 

}
