// @(#)$Id: JMLValueBagSpecs.java,v 1.11 2004/05/21 16:18:52 leavens Exp $

// Copyright (C) 1998, 1999, 2002 Iowa State University

// This file is part of JML

// JML is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.

// JML is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with JML; see the file COPYING.  If not, write to
// the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.


package org.jmlspecs.models;

/** Special behavior for JMLValueBag not shared by JMLObjectBag.
 *
 * @version $Revision: 1.11 $
 * @author Gary T. Leavens, with help from Clyde Ruby, and others.
 * @see JMLValueType
 * @see JMLValueBag
 */
public /*@ pure @*/ abstract class JMLValueBagSpecs implements JMLValueType {

    /** Is the argument ".equals" to one of the values in this bag.
     *  @see #has(Object)
     *  @see #count(JMLType)
     */
    /*@ public normal_behavior
      @   ensures \result <==> count(elem) > 0;
      @*/
    public abstract boolean has(JMLType elem);

    /** Is the argument ".equals" to one of the values in this bag.
     *  @see #has(JMLType)
     *  @see #count(Object)
     */
    /*@   public normal_behavior
      @     requires elem != null;
      @     ensures \result
      @        <==> elem instanceof JMLType && has((JMLType)elem);
      @ also
      @   public normal_behavior
      @     requires elem == null;
      @     ensures \result == has(null);
      @*/    
    public boolean has(Object elem) {
        if (elem == null) {
            return has(null);
        } else {
            return elem instanceof JMLType && has((JMLType)elem);
        }
    }  

    /** Tell many times the argument occurs ".equals" to one of the
     * values in this bag.
     *  @see #count(Object)
     *  @see #has(JMLType)
     */
    /*@ public normal_behavior
      @   ensures \result >= 0
      @       && (* \result is the number of times elem tests
      @              as ".equals" to one of the objects in the bag *);
      @*/
    public abstract int count(JMLType elem);

    /** Tell many times the argument occurs ".equals" to one of the
     * values in this bag.
     *  @see #count(JMLType)
     *  @see #has(Object)
     */
    /*@   public normal_behavior
      @     requires elem != null;
      @     ensures \result
      @          == (elem instanceof JMLType ? count((JMLType)elem) : 0);
      @ also
      @   public normal_behavior
      @     requires elem == null;
      @     ensures \result == count(null);
      @*/    
    public int count(Object elem) {
        if (elem == null) {
            return count(null);
        } else {
            return (elem instanceof JMLType ? count((JMLType)elem) : 0);
        }
    }  

    /** Tells the number of elements in the bag.
     */
    /*@ public normal_behavior
      @   ensures \result >= 0
      @        && (* \result is the number of elements in the bag *);
      @*/
    public abstract int size();

    // Specification inherited from JMLValueType.
    // Note: not requiring the \result contain "clones" of the elements of 
    // this, allows the implementer latitude to take advantage of the
    // nature of immutable types.
    public abstract Object clone();

    /** How many times does the argument occur as an objects
     *  representing values in the bag?
     *  @see #count(Object)
     */
    /*@ public normal_behavior
      @     ensures (* \result is the number of occurences of objects
      @                in this bag that are "==" to elem *);
      public model int countObject(JMLType elem); @*/

    /** Returns a new bag that contains all the elements of this and
     *  also the given argument.
     *  @see #insert(JMLType, int)
     */
    public abstract /*@ non_null @*/ JMLValueBag insert (JMLType elem)
        /*@  public model_program {
          @    assume elem != null && size() < Integer.MAX_VALUE;
          @    JMLType copy = (JMLType) elem.clone();
          @    JMLValueBag returnVal = null;
          @   
          @    behavior
          @      assignable returnVal;
          @      ensures returnVal != null
          @           && (\forall JMLType e; !e.equals(copy);
          @                       countObject(e) == returnVal.countObject(e))
          @           && returnVal.countObject(copy) == 1
          @           && returnVal.size() == size() + 1;
          @
          @      return returnVal; 
          @  }
          @*/
        ;

    /** Returns a new bag that contains all the elements of this and
     *  also the given argument.
     *  @see #insert(JMLType)
     */
    /*@  public model_program {
      @    assume cnt >= 0;
      @    assume elem != null && size() < Integer.MAX_VALUE - cnt;
      @    JMLType copy = (JMLType) elem.clone();
      @    JMLValueBag returnVal = null;
      @   
      @    behavior
      @      assignable returnVal;
      @      ensures returnVal != null
      @           && (\forall JMLType e; e != copy;
      @                       countObject(e)
      @                        == returnVal.countObject(e))
      @           && returnVal.countObject(copy) == cnt
      @           && returnVal.size() == size() + cnt;
      @
      @      return returnVal; 
      @  }
      @
      @ also
      @    signals (IllegalArgumentException) cnt < 0;
      @*/
    public abstract /*@ non_null @*/ JMLValueBag insert (JMLType elem,
                                                         int cnt)
        throws IllegalArgumentException;

}
